/***************************************************************************
                          kemulator.cpp  -  main application
                             -------------------
    begin                : Sun Sep 16 06:29:12 EDT 2001
    copyright            : (C) 2001 by Scott Patterson
    email                : kashis@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qfileinfo.h>
#include <qpixmap.h>
#include <qmessagebox.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>

// application specific includes
#include "kemulator.h"
#include "kemulatorwidget.h"
#include "emulator.h"
#include "mess.h"
#include "messdlgimpl.h"
#include "stella.h"
#include "stelladlgimpl.h"
#include <typeinfo>
#include <iostream.h>

#define ID_STATUS_MSG 1

KemulatorApp::KemulatorApp(QWidget* , const char* name):KMainWindow(0, name)
{
	emulator = 0;
	mess = 0;
	stella = 0;

	config=kapp->config();

	// call inits to invoke all other construction parts
	initStatusBar();
	initActions();
	initCentralWidget();
	readOptions();
}

KemulatorApp::~KemulatorApp()
{
}

void KemulatorApp::initActions()
{
	// Create the actions for menus and toolbars
	fileRandom = new KAction(i18n("&Random"), "unknown", 0, this, SLOT(slotFileRandom()), actionCollection(), "fileRandom");
	fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
	emulatorStella = new KAction(i18n("&Stella"), 0, this, SLOT(slotEmulatorStella()), actionCollection(), "emulatorStella");
	emulatorMess_a5200 = new KAction(i18n("&Mess a5200"), 0, this, SLOT(slotEmulatorMess_a5200()), actionCollection(), "emulatorMess_a5200");
	emulatorMess_a7800 = new KAction(i18n("&Mess a7800"), 0, this, SLOT(slotEmulatorMess_a7800()), actionCollection(), "emulatorMess_a7800");
	emulatorMess_coleco = new KAction(i18n("&Mess coleco"), 0, this, SLOT(slotEmulatorMess_coleco()), actionCollection(), "emulatorMess_coleco");
	emulatorMess_genesis = new KAction(i18n("&Mess genesis"), 0, this, SLOT(slotEmulatorMess_genesis()), actionCollection(), "emulatorMess_genesis");
	emulatorMess_intv = new KAction(i18n("&Mess intv"), 0, this, SLOT(slotEmulatorMess_intv()), actionCollection(), "emulatorMess_intv");
	emulatorMess_nes = new KAction(i18n("&Mess nes"), 0, this, SLOT(slotEmulatorMess_nes()), actionCollection(), "emulatorMess_nes");
	emulatorMess_nespal = new KAction(i18n("&Mess nespal"), 0, this, SLOT(slotEmulatorMess_nespal()), actionCollection(), "emulatorMess_nespal");
	emulatorMess_sms = new KAction(i18n("&Mess sms"), 0, this, SLOT(slotEmulatorMess_sms()), actionCollection(), "emulatorMess_sms");
	emulatorSnes9x = new KAction(i18n("&Snes9x"), 0, this, SLOT(slotEmulatorSnes9x()), actionCollection(), "emulatorSnes9x");
	emulatorMess_ti99_4a = new KAction(i18n("&Mess ti99_4a"), 0, this, SLOT(slotEmulatorMess_ti99_4a()), actionCollection(), "emulatorMess_ti99_4a");
	emulatorMess_vectrex = new KAction(i18n("&Mess vectrex"), 0, this, SLOT(slotEmulatorMess_vectrex()), actionCollection(), "emulatorMess_vectrex");
	emulatorConfigure = new KAction(i18n("&Configure"), "configure", 0, this, SLOT(slotEmulatorConfigure()), actionCollection(), "emulatorConfigure");
	emulatorRefresh = new KAction(i18n("&Refresh"), "reload", 0, this, SLOT(slotEmulatorRefresh()), actionCollection(), "emulatorRefresh");
	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
	viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

	// Set the action texts
	fileRandom->setStatusText(i18n("Select a ROM at random"));
	fileQuit->setStatusText(i18n("Quits the application"));
	emulatorStella->setStatusText(i18n("Loads Stella ROM list"));
	emulatorMess_a5200->setStatusText(i18n("Loads MESS a5200 ROM list"));
	emulatorMess_a7800->setStatusText(i18n("Loads MESS a7800 ROM list"));
	emulatorMess_coleco->setStatusText(i18n("Loads MESS coleco ROM list"));
	emulatorMess_genesis->setStatusText(i18n("Loads MESS genesis ROM list"));
	emulatorMess_intv->setStatusText(i18n("Loads MESS intv ROM list"));
	emulatorMess_nes->setStatusText(i18n("Loads MESS nes ROM list"));
	emulatorMess_nespal->setStatusText(i18n("Loads MESS nespal ROM list"));
	emulatorMess_sms->setStatusText(i18n("Loads MESS sms ROM list"));
	emulatorSnes9x->setStatusText(i18n("Loads Snes9x ROM list"));
	emulatorMess_ti99_4a->setStatusText(i18n("Loads MESS ti99_4a ROM list"));
	emulatorMess_vectrex->setStatusText(i18n("Loads MESS vectrex ROM list"));
	emulatorConfigure->setStatusText(i18n("Configure the emulator"));
	emulatorRefresh->setStatusText(i18n("Refresh the emulator"));
	viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
	viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

	// use the absolute path to your kemulatorui.rc file for testing purpose in createGUI();
	createGUI();
}


void KemulatorApp::initStatusBar()
{
	statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KemulatorApp::saveOptions()
{	
	config->setGroup("General Options");
	config->writeEntry("Geometry", size());
	config->writeEntry("Show Toolbar", viewToolBar->isChecked());
	config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
	config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
}


void KemulatorApp::initCentralWidget()
{
	kemulatorWidget = new KemulatorWidget(this, "kemulatorWidget");
	setCentralWidget(kemulatorWidget);	
}


void KemulatorApp::readOptions()
{
	config->setGroup("General Options");

	// bar status settings
	bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
	viewToolBar->setChecked(bViewToolbar);
	slotViewToolBar();

	bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
	viewStatusBar->setChecked(bViewStatusbar);
	slotViewStatusBar();

	// bar position settings
	KToolBar::BarPosition toolBarPos;
	toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
	toolBar("mainToolBar")->setBarPos(toolBarPos);
	
	QSize size=config->readSizeEntry("Geometry");
	if(!size.isEmpty())
	{
		resize(size);
	}
}

bool KemulatorApp::queryExit()
{
	saveOptions();
	return true;
}

void KemulatorApp::deleteEmulator()
{
	if (mess != 0) 
	{
		delete mess;
		emulator = 0;
		mess = 0;
	}
/*
	if (snes9x != 0) 
	{
		delete snes9x;
		emulator = 0;
		snes9x = 0;
	}
*/
	if (stella != 0) 
	{
		delete stella;
		emulator = 0;
		stella = 0;
	}
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KemulatorApp::slotFileRandom()
{
	slotStatusMsg(i18n("Selecting..."));
	kemulatorWidget->random();
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotFileQuit()
{
	slotStatusMsg(i18n("Exiting..."));
	saveOptions();
	slotStatusMsg(i18n("Ready."));
	close();
}

void KemulatorApp::slotEmulatorStella()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	stella = new Stella();
	emulator = stella;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_a5200()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("a5200");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_a7800()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("a7800");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_coleco()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("coleco");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_genesis()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("genesis");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_intv()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("intv");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_nes()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("nes");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_nespal()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("nespal");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_sms()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("sms");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorSnes9x()
{
	slotStatusMsg(i18n("Loading..."));
	//deleteEmulator();
	//snes9x = new Snes9x();
	//emulator = mess;
	//kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_ti99_4a()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("ti99_4a");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorMess_vectrex()
{
	slotStatusMsg(i18n("Loading..."));
	deleteEmulator();
	mess = new Mess("vectrex");
	emulator = mess;
	kemulatorWidget->insertStringList(emulator->romList());
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorConfigure()
{
	slotStatusMsg(i18n("Configure..."));

	if (emulator == 0)
	{
		QMessageBox::warning(this, i18n("Configuration..."),
		                     i18n("Please select an emulator."));
	}
	else
	{
		if (typeid(*emulator) == typeid(Mess))
		{
			MessDlgImpl messDlgImpl(mess->system(), this, "messDlgImpl", true);
			messDlgImpl.exec();
			// Reload the configuration if saved
			if (messDlgImpl.result() != QDialog::Rejected) slotEmulatorRefresh();
		}
/*
		if (typeid(*emulator) == typeid(Snes9x))
		{
			Snes9xDlgImpl stellaDlgImpl(this, "snes9xDlgImpl", true);
			snes9xDlgImpl.exec();
			// Reload the configuration if saved
			if (snes9xDlgImpl.result() != QDialog::Rejected) slotEmulatorRefresh();
		}
*/
		if (typeid(*emulator) == typeid(Stella))
		{
			StellaDlgImpl stellaDlgImpl(this, "stellaDlgImpl", true);
			stellaDlgImpl.exec();
			// Reload the configuration if saved
			if (stellaDlgImpl.result() != QDialog::Rejected) slotEmulatorRefresh();
		}
	}

	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotEmulatorRefresh()
{
	slotStatusMsg(i18n("Loading..."));
	if (emulator != 0) 
	{
		emulator->refresh();
		kemulatorWidget->insertStringList(emulator->romList());
	}
	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotViewToolBar()
{
	slotStatusMsg(i18n("Toggling toolbar..."));

	///////////////////////////////////////////////////////////////////
	// turn Toolbar on or off
	if(!viewToolBar->isChecked())
	{
		toolBar("mainToolBar")->hide();
	}
	else
	{
		toolBar("mainToolBar")->show();
	}		

	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotViewStatusBar()
{
	slotStatusMsg(i18n("Toggle the statusbar..."));

	///////////////////////////////////////////////////////////////////
	//turn Statusbar on or off
	if(!viewStatusBar->isChecked())
	{
		statusBar()->hide();
	}
	else
	{
		statusBar()->show();
	}

	slotStatusMsg(i18n("Ready."));
}

void KemulatorApp::slotStatusMsg(const QString &text)
{
	// change status message permanently
	statusBar()->clear();
	statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KemulatorApp::slotSelected(int index)
{
	emulator->emulate(index);
}
